package ru.ulstu.extractor.gitrepository.service;

import org.json.JSONArray;
import org.springframework.stereotype.Service;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.http.HttpService;

import static ru.ulstu.extractor.util.StringUtils.getUrlParts;
import static ru.ulstu.extractor.util.StringUtils.removeDotGit;

@Service
public class GitlabApi implements GitApi {
    private final HttpService httpService;
    private static final String BRANCHES_COUNT_URL = "%s/api/v4/projects/%s/repository/branches";
    private final static String PROJECT_ID_URL = "https://gitlab.com/api/v4/users/%s/projects?search=%s";
    private static final String PROJECT_INFO_URL = "%s/api/v4/projects/%s";
    private static final String OPEN_ISSUES_URL = "%s/api/v4/projects/%s/issues?state=opened";
    private static final String COMPLETED_ISSUES_URL = "%s/api/v4/projects/%s/issues?state=closed";

    public GitlabApi(HttpService httpService) {
        this.httpService = httpService;
    }

    @Override
    public Integer getBranchesCount(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, BRANCHES_COUNT_URL))
                .length();
    }

    @Override
    public Integer getStarsCount(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, PROJECT_INFO_URL))
                .getJSONObject(0)
                .getInt("star_count");
    }

    @Override
    public Integer getOpenIssuesCount(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, OPEN_ISSUES_URL))
                .length();
    }

    @Override
    public String getFormattedUrl(Branch branch, String template) {
        String[] urlParts = getUrlParts(branch.getGitRepository().getUrl());
        return String.format(template, urlParts[0] + "/" + urlParts[1] + "/" + urlParts[2], branch.getGitRepository().getRepositoryId());
    }

    public String getFormattedUrlForProjectId(String repositoryUrl, String template) {
        String[] urlParts = getUrlParts(repositoryUrl);
        // получаем корректное название репозитория, gitlab всегда добавляет .git в конец
        return String.format(template, urlParts[3], removeDotGit(urlParts[4]));
    }

    @Override
    public Integer getAuthorsCompletedIssues(Branch branch) {
        return httpService
                .get(getFormattedUrl(branch, COMPLETED_ISSUES_URL))
                .length();
    }

    public String getRepositoryId(String repositoryUrl) {
        JSONArray projects = httpService.get(getFormattedUrlForProjectId(repositoryUrl, PROJECT_ID_URL));
        for (int i = 0; i < projects.length(); i++) {
            if (projects.getJSONObject(i).get("name").equals(removeDotGit(getUrlParts(repositoryUrl)[4]))) {
                return String.valueOf(projects.getJSONObject(i).getInt("id"));
            }
        }
        throw new RuntimeException("Id проекта не найден: " + repositoryUrl);
    }
}
